"""J.A.R.V.I.S. Utility Functions."""
from datetime import datetime
from pkgutil import iter_modules

import git
from dis_snek.models.discord.embed import Embed

import jarvis.cogs
import jarvis.db
from jarvis.config import get_config

__all__ = ["cachecog", "permissions"]


def build_embed(
    title: str,
    description: str,
    fields: list,
    color: str = "#FF0000",
    timestamp: datetime = None,
    **kwargs: dict,
) -> Embed:
    """Embed builder utility function."""
    if not timestamp:
        timestamp = datetime.now()
    embed = Embed(
        title=title,
        description=description,
        color=color,
        timestamp=timestamp,
        **kwargs,
    )
    for field in fields:
        embed.add_field(**field.to_dict())
    return embed


def get_extensions(path: str = jarvis.cogs.__path__) -> list:
    """Get J.A.R.V.I.S. cogs."""
    config = get_config()
    vals = config.cogs or [x.name for x in iter_modules(path)]
    return ["jarvis.cogs.{}".format(x) for x in vals]


def update() -> int:
    """J.A.R.V.I.S. update utility."""
    repo = git.Repo(".")
    dirty = repo.is_dirty()
    current_hash = repo.head.object.hexsha
    origin = repo.remotes.origin
    origin.fetch()
    if current_hash != origin.refs["main"].object.hexsha:
        if dirty:
            return 2
        origin.pull()
        return 0
    return 1


def get_repo_hash() -> str:
    """J.A.R.V.I.S. current branch hash."""
    repo = git.Repo(".")
    return repo.head.object.hexsha
